#include <sstream>
#include <string>
#include <curl/curl.h>
#include <iostream>
#include <vector>
#include "histogram.h"
#include "text.h"
#include "svg.h"
using namespace std;
struct Input
{
    vector<double> numbers;
    size_t bin_count{};
    size_t number_count{};
    size_t max_count{};
};
Input
input_data(istream& cin, bool prompt)
{
    if (prompt==true)
    {
        Input in;
        cerr << "Enter number count: ";
        cin >> in.number_count;
        vector<double> numbers(in.number_count);
        in.numbers.resize(in.number_count);
        for (size_t i = 0; i < in.number_count; i++)
        {
            cin >> in.numbers[i];
        }
        size_t bin_count;
        cerr << "Enter bin count: ";
        cin >> in.bin_count;

        size_t max_count;
        in.max_count = 0;
        return in;
    }
    if (prompt==false)
    {
        Input in;
        cin >> in.number_count;
        vector<double> numbers(in.number_count);
        in.numbers.resize(in.number_count);
        for (size_t i = 0; i < in.number_count; i++)
        {
            cin >> in.numbers[i];
        }
        size_t bin_count;
        cin >> in.bin_count;

        size_t max_count;
        in.max_count = 0;
        return in;
    }
}
size_t
write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
    size_t data_size = item_size * item_count;
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}
Input
download(const string& address)
{
    stringstream buffer;
    CURL* curl = curl_easy_init();
    if(curl)
    {
        CURLcode res;
        const char* res2;
        double connect;
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        res = curl_easy_perform(curl);
        if (res!=0)
        {
            res2=curl_easy_strerror(res);
            cout<<res2<<endl;
            exit(1);
        }

        if(CURLE_OK == res) {
      res = curl_easy_getinfo(curl, CURLINFO_CONNECT_TIME, &connect);
      if(CURLE_OK == res) {
        cerr<<("Time: %.1f", connect)<<endl;
      }
    }
        curl_easy_cleanup(curl);
    }
    curl_global_init(CURL_GLOBAL_ALL);
    return input_data(buffer, false);
}

int main(int argc, char* argv[])
{
    Input in;
    if (argc > 1)
    {
        in = download(argv[1]);
    }
    else
    {
        in = input_data(cin, true);
    }

    auto bins = make_histogram(in.numbers, in.bin_count, in.number_count, in.max_count);
    show_histogram_svg(bins, in.max_count, in.bin_count);
    return 0;
}
